#data folder
setwd("~/surfdrive/CYPSY23_eHealth4MDD_analyses/Analyses") 
path <- "./analysis/"

# save output to file
sink(paste0(path,"output_CYPSY23_tables.txt"))
sessionInfo()

cat("#########################################################################################################\n")
cat("# Analysis of reliability concerning mapping of functions to therapy classes, identifying functions in\n")
cat("# articles, and classifying function instantiations as specific function from article snippets.\n") 
cat("# 2018 TUDelft\n")
cat("# Author: Franziska Burger\n")
cat("# Date: July 2018\n")
cat("# Output files: output_CYPSY23_tables.txt\n")
cat("# Required files:\n")
cat("# - literature_screening_C1C2.txt\n")
cat("# - function_identification_C1C3.txt\n")
cat("# - function_classification_C1C3\n")
cat("# - function_therapy_C1C3.txt\n")
cat("# - validation_C1CX.txt\n")
cat("# #########################################################################################################\n")
cat("\n")
cat("\n")
cat("################################# LIBRARIES ###################\n")
require(plyr)
require(dplyr)
require(tidyr)
require(reshape)
require(RMySQL)
require(Hmisc)
require(boot)
require(psych)
require(irr)
cat("# plyr\n# dplyr\n# tidyr\n# reshape\n# RMySQL\n# Hmisc\n# boot\n# psych\n# irr\n")
cat("#################################\n")
cat("\n")
cat("\n")
cat("################################# DATA ###################\n")
cat("########## literature_screening_C1C2.txt\n")
cat("# Contains the results of the title, abstract, and full paper filtering of records as done by coder 1 (C1) and coder 2 (C2)\n")
cat("#\n")
cat("# data fields\n")
cat("#\n")
cat("# PaperKey: key of the record as assigned by endnote on computer of C1\n")
cat("# C1: coding assigned by coder 1\n")
cat("# C2: coding assigned by coder 2\n")
cat("# Stage: which stage of the filtering process the record was evaluated at (either title, abstract, or full paper stage)\n")

cat("#################################\n")
cat("########## function_identification_C1C3.txt\n")   
cat("# File with ID of instantiation of a function (brief description taken from articles) and the function key that coder 1 (C1) and coder 3 (C3) assigned to this instantiation\n")
cat("#\n")
cat("# data fields\n")
cat("#\n")
cat("#1 VersionKey: unique identifier for each ehealth system version used in this reliability check\n")
cat("#2 FunctionKey: unique identifier for a specific function that was found to be in version by either C1 or C3 \n")
cat("#3 C1: 1 if C1 assigned this function, na if C1 did not assign this function\n")
cat("#4 C3: 1 if C3 assigned this function, na if C3 did not assign this function\n")
cat("#\n")
cat("########## function_classification_C1C3.txt\n")   
cat("# File with ID of instantiation of a function (brief description taken from articles) and the function key that C1 and C3 assigned to this instantiation\n")
cat("#\n")
cat("# data fields\n")
cat("#\n")
cat("#1 ID: unique identifier for each function instantiation\n")
cat("#2 C1: the function key of the corresponding function as assigned by C1\n")
cat("#3 C3: the function key of the corresponding function as assigned by C3\n")
cat("#\n")
cat("########## function_therapy_mapping_C1C3.txt\n")   
cat("# File with mapping of therapy intervention class (e.g. CBT, CT, ACT, psychodynamic psychotherapy, ...) to functions\n")
cat("#\n")
cat("# data fields\n")
cat("#\n")
cat("#1 FunctionKey: unique identifier for each intervention function of an ehealth system\n")
cat("#2 Description: a brief description of the functionality provided by each function \n")
cat("#3 C1: the respective code for a therapy intervention class assigned to the function by C1\n")
cat("#4 C3: the list of codes of therapy intervention classes assigned to the function by C3\n")
cat("#\n")
cat("#Codes for the therapy intervention classes are as follows:\n")
cat("#1. CBT\n") 
cat("#2. cognitive therapy\n")
cat("#3. behavioral activation\n")
cat("#4. motivational interviewing\n")
cat("#5. cognitive bias modification\n")
cat("#6. problem solving\n")
cat("#7. ACT\n")
cat("#8. bibliotherapy\n")
cat("#9. psychoeducation\n")
cat("#10.positive psychology\n")
cat("#11.mindfulness\n")
cat("#12.physical activity\n")
cat("#13.interpersonal therapy\n")
cat("#14.psychodynamic psychotherapy\n")
cat("#15.emotional freedom technique\n")
cat("#16.life review\n")
cat("#17.expressive writing\n")
cat("#19.integrative/multiple\n")
cat("#20.schema therapy\n")
cat("#21.meta-cognitive\n")
cat("#22.social cognitive theory\n")
cat("#23.cognitive control training\n")
cat("#24.hypnosis\n")
cat("########## validation_C1CX.txt\n")   
cat("# File with the assigned ehdts scores of coders 2, and 4-8 when they did not know what each number of the scale meant, i.e. when they were asked to\n")
cat("# rate function instantiations according to their intuition of technological sophistication, when coders did know the definitions of scale levels,\n")
cat("# and the assigned scores by C1 to the same items")
cat("#\n")
cat("# data fields\n")
cat("#\n")
cat("#1 VersionKey: unique identifier for the Version\n")
cat("#2 FunctionKey: unique identifier for the function, together with the VersionKey this identifies the item that was rated\n")
cat("#3 CXnoscale: eHDTS score as assigned by CX (where X is element of {4,5,6,7,8}) without knowing the definitions of the scale levels\n")
cat("#4 CXscale: eHDTS score as assigned by CX when knowing the definitions of the scale levels\n")
cat("#5 C1scale: eHDTS score as assigned by C1 when knowing the definitions of the scale levels\n")
cat("#6 coderID: ID of the coder that completed the coding (e.g. 4 -> C4)\n")
cat("#\n")
cat("#################################\n")
cat("\n")
cat("\n")

################################# Functions #################################
########### Validity
#computes the spearman correlation between two variables of a dataframe, the corrs function is needed for the bootstrapping below
corrs <- function(df,name1, name2, indices){
  if(missing(indices)){
    return(cor(df[,c(name1,name2)],method="spearman",use="complete.obs")[name1,name2])
  }
  else {
    return(cor(df[indices,c(name1,name2)],method="spearman",use="complete.obs")[name1,name2])
  }
}

agrees <- function(df,name1,name2,indices){
  return(agree(df[indices,c(name1,name2)])$value)
}

#spearman correlations of intra-coder ratings (naive vs informed) and intercoder rating (CX informed vs C1 informed) and bootstrapped CIs
correlations <- function(df,coderID){
  intracor <- corrs(df[which(df$coderID==coderID),],"CXnoscale","CXscale")
  intracor.bs <- boot(data=df[which(df$coderID==coderID),c("CXnoscale","CXscale")], statistic=corrs, R=10000, name1="CXnoscale", name2="CXscale")
  intracor.cis <- boot.ci(intracor.bs, type="bca")
  intracor.cis <- intracor.cis$bca[1,c(4,5)]
  intercor <- corrs(df[which(df$coderID==coderID),],"C1scale","CXscale")
  intercor.bs <- boot(data=df[which(df$coderID==coderID),c("C1scale","CXscale")], statistic=corrs, R=10000, name1="C1scale",name2="CXscale")
  intercor.cis <- boot.ci(intercor.bs, type="bca")
  intercor.cis <- intercor.cis$bca[1,c(4,5)]
  intra <- c(intracor, intracor.cis)
  inter <- c(intercor, intercor.cis)
  info <- c("estimate","BS_lower-CI","BS_upper-CI")
  df.corrs <- data.frame(info,intra,inter)
  return(df.corrs)
}


################################# READING IN DATA #################################
########### Literature Screening
df.litscreen <- read.table(file=paste0(path,"literature_screening_C1C2.txt"), header=TRUE, sep="\t", fill=TRUE)

########### High-Inference Variables
df.identifying <- read.table(paste0(path,"function_identification_C1C3.txt"), header=TRUE, sep="\t", fill=TRUE)

df.classification <- read.table(paste0(path,"function_classification_C1C3.txt"), header=TRUE, sep="\t")

df.mapping <- read.table(paste0(path,"function_therapy_mapping_C1C3.txt"), header=TRUE, sep="\t")

########### Validity
df.validation <- read.table(paste0(path,"validation_C1CX.txt"), header=TRUE, sep="\t")
cat("\n")
cat("\n")
cat("\n")

cat("################################# Analyses Table 1 #################################\n")
cat("########### Literature Screening\n")
cat("###Reliability analysis\n")
cat("##title testing\n")
df.title <- df.litscreen[which(df.litscreen$Stage=="title"),c("C1","C2")]
cat("Population: \n")
print(4266)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on title alone\n")
agree_tite <- agree(df.title)
print(agree_tite)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement on title\n")
title.bs <- boot(data=df.title, statistic=agrees, R=10000, name1="C1", name2="C2")
title.cis <- boot.ci(title.bs, type="bca")
print(title.cis)
cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on title\n")
kappa_tite <- kappa2(df.title, weight = "unweighted")
print(kappa_tite)
cat("\n")
cat("confidence intervals for unweighted Cohen's kappa for agreement on title\n")
kappa_tite_2 <- cohen.kappa(df.title, w=NULL, alpha=.05)
print(kappa_tite_2$confid[1,])
cat("\n")
cat("##abstract testing\n")
df.abstract <- df.litscreen[which(df.litscreen$Stage=="abstract"),c("C1","C2")]
cat("Population:\n")
print(1071)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on abstract\n")
agree_abte <- agree(df.abstract)
print(agree_abte)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement on abstract\n")
abstract.bs <- boot(data=df.abstract, statistic=agrees, R=10000, name1="C1", name2="C2")
abstract.cis <- boot.ci(abstract.bs, type="bca")
print(abstract.cis)
cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on abstract\n")
kappa_abte <- kappa2(df.abstract, weight = "unweighted")
print(kappa_abte)
cat("\n")
cat("#confidence intervals for unweighted Cohen's kappa for agreement on abstract\n")
kappa_abte_2 <- cohen.kappa(df.abstract, w=NULL, alpha=.05)
print(kappa_abte_2$confid[1,])
cat("\n")
cat("##full-paper testing\n")
df.full <- df.litscreen[which(df.litscreen$Stage=="fullpaper"),c("C1","C2")]
cat("Population: \n")
print(541)
cat("#percentage agreement of C1 and C2 on deciding to include a record based on full article\n")
agree_fute <- agree(df.full)
print(agree_fute)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement of full-paper\n")
full.bs <- boot(data=df.full, statistic=agrees, R=10000, name1="C1", name2="C2")
full.cis <- boot.ci(full.bs, type="bca")
print(full.cis)
cat("\n")
cat("#unweighted Cohen's kappa and significance test of agreement on full paper\n")
kappa_fute <- kappa2(df.full, weight = "unweighted")
print(kappa_fute)
cat("\n")
cat("#confidence intervals for unweighted Cohen's kappa for agreement on full paper\n")
kappa_fute_2 <- cohen.kappa(df.full, w=NULL, alpha=.05)
print(kappa_fute_2$confid[1,])
cat("\n")
cat("\n")
cat("\n")


cat("########### High-Inference Variables\n")
cat("### IDENTIFY FUNCTIONS OF SYSTEMS\n")
cat("#mismatch in population between table in paper and number of subjects reported here stems from the way we analyzed the agreement. 10 papers\n")
cat("were provided to C3 (n reported in paper) but for the analysis we pooled all identified functions by C1 and C3 (in total 208)\n")
cat("and counted as agreement when both C1 and C3 identified a function to be in a given paper and as disagreement when only C1 or C3 identified\n")
cat("the function.\n")
cat("Population: \n")
print(273)
cat("#percent agreement in identifying a function from a textual version description\n")
percagg_identifying <- agree(df.identifying[,c("C1","C3")])
print(percagg_identifying)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement of system-function-identification\n")
sfi.bs <- boot(data=df.identifying[,c("C1","C3")], statistic=agrees, R=10000, name1="C1", name2="C3")
sfi.cis <- boot.ci(sfi.bs, type="bca")
print(sfi.cis)
cat("\n")
cat("\n")
cat("### CLASSIFY FUNCTIONS OF SYSTEMS\n")
cat("Population: \n")
print(2224)
cat("#percent agreement of C1 and C3 in classifying function instantiations as instantiations of a specific function:\n") 
percagg_classification <- agree(df.classification[,c("C1","C3")])
print(percagg_classification)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement of function-instantiation-classification\n")
fic.bs <- boot(data=df.classification[,c("C1","C3")], statistic=agrees, R=10000, name1="C1", name2="C3")
fic.cis <- boot.ci(fic.bs, type="bca")
print(fic.cis)
cat("\n")
cat("#Unweighted Cohen's kappa between C1 and C3 and significance test:\n") 
c_kappa_classification <- kappa2(df.classification[,c("C1","C3")], weight = "unweighted")
print(c_kappa_classification)
cat("\n")
cat("#Confidence intervals for the unweighted Cohen's kappa:\n")
c_kappa2_classification <- cohen.kappa(df.classification[,c("C1","C3")], w=NULL, alpha=.05)
print(c_kappa2_classification$confid[1,])
cat("\n")
cat("\n")
cat("### MAP INTERVENTION FUNCTIONS TO THERAPEUTIC INTERVENTION CLASSES\n")
cat("Population: \n")
print(141)
cat("#percent agreement in mapping a function to a therapeutic intervention class\n")
percagg_therapymapping <- agree(df.mapping[,c("C1","C3")])
print(percagg_therapymapping)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement of function-therapy-mapping\n")
ftm.bs <- boot(data=df.mapping[,c("C1","C3")], statistic=agrees, R=10000, name1="C1", name2="C3")
ftm.cis <- boot.ci(ftm.bs, type="bca")
print(ftm.cis)
cat("\n")
cat("\n")
cat("### ASSIGN DEGREE OF TECHNOLOGICAL SOPHISTICATION TO INSTANTIATION\n")
cat("Population:\n")
print(2224)
cat("#percent agreement and cohen's kappa for automation degree assigned to all functions by C2\n")
df.val <- df.validation[which(df.validation$coderID==2 & !is.na(df.validation$CXscale)),c("CXscale","C1scale")]
perc_agg <- agree(df.val)
print(perc_agg)
cat("\n")
cat("#bootstrapped confidence intervals for percent agreement of eHDTS-scoring\n")
ehdtss.bs <- boot(data=df.val, statistic=agrees, R=10000, name1="CXscale", name2="C1scale")
ehdtss.cis <- boot.ci(ehdtss.bs, type="bca")
print(ehdtss.cis)
cat("\n")
cat("#weighted Cohen's Kappa and significance test:\n")
c_kappa <- kappa2(df.val, weight = "squared")
print(c_kappa)
cat("\n")
cat("#weighted Cohen's Kappa with confidence intervals:\n")
c_kappa2 <- cohen.kappa(df.val, alpha=.05)
print(c_kappa2$confid[2,])
cat("\n")
cat("\n")
cat("\n")


cat("\n")
cat("\n")
cat("\n")
cat("################################# Analyses Table 2 #################################\n")
cat("########### Validity\n")
cat("##INTRA-CODER Correlations: each coder coded the items once when unfamiliar with the scale levels and once when familiar\n") 
cat("##with the scale levels. Intracoder correlations correlate these two scores (naive and informed). Confidence intervals are obtained by
    bootstrapping.\n")
cat("##INTER-CODER Correlation: the codes assigned by the informed second coders are correlated with the informed codes assigned by C1.\n")
cat("#intervention correlation C4 naive with informed (intra) and informed C4 with informed C1 (inter)\n")
cat("Population:\n")
print(1344)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==4),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,4))
cat("\n")

cat("#planning correlation C5 naive with informed (intra) and informed C5 with informed C1 (inter)\n")
cat("Population:\n")
print(29)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==5),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,5))
cat("\n")

cat("#execution correlation C6 naive with informed (intra) and informed C6 with informed C1 (inter)\n")
cat("Population:\n")
print(445)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==6),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,6))
cat("\n")

cat("#monitoring correlation C7 naive with informed (intra) and informed C7 with informed C1 (inter)\n")
cat("Population:\n")
print(140)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==7),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,7))
cat("\n")

cat("#social correlation C8 naive with informed (intra) and informed C8 with informed C1 (inter)\n")
cat("Population:\n")
print(266)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==8),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,8))
cat("\n")

cat("#all items mixed correlation of C2 naive versus informed scores (intra) and informed C2 with informed C1 (inter)\n")
cat("Population:\n")
print(2224)
cat("Sample:\n")
print(nrow(df.validation[which(df.validation$coderID==2 & (!is.na(df.validation$CXnoscale) & !is.na(df.validation$CXscale))),]))
cat("Coders:\n")
print(2)
cat("Correlations with bootstrapped confidence intervals (bootstrap samples=10000):\n")
print(correlations(df.validation,2))
cat("\n")

sink()