#data folder
path <- "./preprocessing_and_raw_data/function_identifying/"

#########################################################################################################
# Preprocessing for the reliability analysis of splitting an ehealth system into its functional components 
# and mapping functions to descriptions as extracted from articles. 
# 2018 TUDelft
# Author: Franziska Burger
# Output file: 
# - function_identification_C1C3.txt
# Required files: 
# - ehealth4mdd_CYPSY23 database
# - identifying_C3.txt
#########################################################################################################

################################# LIBRARIES ###################
require(dplyr)
require(plyr)
require(tidyr)
require(reshape)
require(irr)
require(stringr)
require(reshape2)
require(ggplot2)
require(RMySQL)
require(magrittr)
#################################

################################# DATA ###################

########## input: 
# - VersionFunction table of ehealth4mdd database
# Contains the splitting up of systems/versions into functions as done by C1
# For more details please visit the database website.
# - identifying_C3.txt
# Contains the splitting up of systems/versions into functions as done by C3
#
# Data Fields:
#
#1 Articles: unique identifier for an article in which a version of an ehealth system is described
#2 Functions: a list of the functions that C3 identified to be in the system described in the article

#reading in C3's function list
df.identifyingC3 <- read.table(paste0(path,"identifying_C3.txt"), header=TRUE, sep="\t", fill=TRUE)

##functions
split_list <- function(df,column,ids){
  #some versions have multiple therapy types or technologies separated by commas or semicolons, here we split these up and create new rows
  #remove spaces
  df[,c(column)] <- lapply(df[,c(column)], function(x){gsub(" ","",x)}) %>% unlist(.)
  #find length of string with most elements (counting commas/semicolons), e.g. "1,8,9" is longer than "7,15"
  n <- df[,c(column)] %>% lapply(function(x){ifelse(is.na(x),0,str_count(x,",|;"))}) %>% unlist(.) %>% max(.)
  n <- n+1
  #expand string into new columns
  df.split <- separate_(df, col=column, into=as.character(c(1:n)), sep=",|;")
  #melt dataframe again so that it has the new columns as new rows
  df.melted <- melt(df.split[,c(ids,as.character(c(1:n)))],id=ids,na.rm=TRUE)
  df.melted$variable <- NULL
  df.melted$value <- factor(df.melted$value)
  return(df.melted)
}

df.identifyingC3_split <- split_list(df.identifyingC3,"Functions","Articles")

#connecting to the ehealth4mdd database
u <- 'root'
p <- 'insy'
n <- 'ehealth4mdd_CYPSY23'
h <- 'localhost'
ehealthdb = dbConnect(MySQL(), user=u, password=p, dbname=n, host=h)

#table that links version keys to article keys
query_vp = dbSendQuery(ehealthdb, 'select * from VersionPaper;')
df.vp <- fetch(query_vp,n=-1)

#now we merge split df with vp df so that we get functions linked to versions instead of articles
df.splitC3 <- merge(df.identifyingC3_split,df.vp, by.x="Articles", by.y="PaperKey",all.x=TRUE)
colnames(df.splitC3)[colnames(df.splitC3) == 'value'] <- 'FunctionKey'

#then we get from the database a table of VersionKeys and FunctionKeys as identified by C1
query_vc = dbSendQuery(ehealthdb, 'select vc.VersionKey, vc.FunctionKey from VersionFunction as vc;')
df.vc <- fetch(query_vc,n=-1)

#we subset those versions that are also coded by C3
df.splitC1 <- df.vc[which(df.vc$VersionKey %in% unique(df.splitC3$VersionKey)),]

#we add a column of 1s to each of the dfs by the coders, because each coder included a specific function
df.splitC1$C1 <- 1
df.splitC3$C3 <- 1

#now we can merge the two dataframes on the VersionKeys and FunctionKeys and export them to a txt file
df.identifyingC1C3 <- merge(df.splitC1,df.splitC3[,c("VersionKey","FunctionKey","C3")], by=c("VersionKey","FunctionKey"), all=TRUE)
#replace NAs by 0 since NA indicates that the coder did not identify a function as part of a Version.
df.identifyingC1C3$C1[is.na(df.identifyingC1C3$C1)] <- 0
df.identifyingC1C3$C3[is.na(df.identifyingC1C3$C3)] <- 0

write.table(df.identifyingC1C3,"analysis/function_identification_C1C3.txt", sep='\t', row.names=FALSE)
