#set working directory to directory of R script
path <- "./preprocessing_and_raw_data/function_therapy_mapping/"

#########################################################################################################
# Preprocessing of raw data of mapping functions to their therapeutic intervention classes as done by coders 1 (C1) and 3 (C3)
# February 2018 TUDelft
# Author: Franziska Burger
# Output files:
# - function_therapy_mapping_C1C3.txt
# Required files:
# - function_therapy_C1C3.txt
#########################################################################################################

################################# LIBRARIES ###################
require(magrittr)
require(plyr)
require(dplyr)
require(stringr)
require(tidyr)
require(reshape)

################################# DATA ###################
########## function_therapy_C1C3.txt
# Contains the results of the mapping of functions to therapeutic frameworks as conducted by C3 merged with the codes assigned by C1
#
# data fields
#
# FunctionKey: unique identifier of the function
# Description: textual description of the function
# Therapy.C1: the therapeutic intervention framework that had been assigned to this function by C1
# Therapy.C3: the therapeutic intervention framework that had been assigned to this function by C3
# Comments: comments made in the file by C3 while coding
# Discussion: points of discussion when iterating over the unclear ones together.

################################# FUNCTIONS ###################
#some versions have multiple therapy types or technologies separated by commas or semicolons, here we split these up and convert into a list of numbers
##read in the categorization of C1 and C3 as determined after discussion.
function_therapy <- read.table(file=paste0(path,"function_therapy_C1C3.txt"), header=TRUE, sep="\t", fill=TRUE)
#split this into two separate dataframes, one for C1 and one for C3
funcsC1 <- function_therapy[,c("FunctionKey","Therapy.C1")]
funcsC3 <- function_therapy[,c("FunctionKey","Therapy.C3")]
funcsC1$Therapy.C1 <- as.character(funcsC1$Therapy.C1) 
funcsC3$Therapy.C3 <- as.character(funcsC3$Therapy.C3) 

#replace lists as strings with lists as numbers
funcsC3$Therapy.C3 <- lapply(funcsC3$Therapy.C3, function(x){as.numeric(as.character(unlist(strsplit(gsub(" ","",x), split=","))))})

##cleaning and assimilating
#C1 always assigned only one therapy to each function while C3 was allowed to assign a list.
#We now reduce the list of C3 to single values according to the following rules 
#First we assign 0 to all values, at the end of this no 0s should remain.
funcsC3$singleC3 <- 0
funcs <- merge(funcsC3, funcsC1, by="FunctionKey")
funcs$Therapy.C1 <- as.numeric(as.character(funcs$Therapy.C1))
#For clarity and legibility, we have split each operation into a separate function rather than putting it into one big loop with comments.
#C3 understood as bibliotherapy (8) the same thing as C1 understood as psychoeducation (9) 
funcs$singleC3 <- apply(funcs, 1, function(x){ifelse(8 %in% x$Therapy.C3 && x$Therapy.C1==9, 9, x$singleC3)})
#If the values assigned by C3 contain the value assigned by C1, we reduce the list of C3 to only this value.
funcs$singleC3 <- apply(funcs, 1, function(x){ifelse(x$singleC3==0 && x$Therapy.C1 %in% x$Therapy.C3, x$Therapy.C1, x$singleC3)})
#There were three codes for CBT related therapies: 1-CBT, 2-CT, 3-BA. Thus, C2 assigned either 2 or 3 while C3 assigned 1, it is agreement. 
#Vice versa, if C1 assigned 1 and C3 assigned 2,3, or both, we also count this as agreement. 
funcs$singleC3 <- apply(funcs, 1, function(x){ifelse(x$singleC3==0 && 
                                                       ((x$Therapy.C1 %in% c(2,3) && 1 %in% x$Therapy.C3) || 
                                                          length(intersect(c(2,3),x$Therapy.C3))==0 && x$Therapy.C1==1), x$Therapy.C1, x$singleC3)})
#Finally, C1 was allowed to assign 19 (multiple therapy classes), while C3 was asked to specifically name all therapy classes.If C3 assigned more than 3 different 
#therapy classes to a function, we code this as 'multiple' (19).
funcs$singleC3 <- apply(funcs, 1, function(x){ifelse(x$singleC3==0 && (x$Therapy.C1==19 && length(x$Therapy.C3)>3), x$Therapy.C1, x$singleC3)})
#all remaining 0 values get assigned a random value from the list that C3 assigned, to indicate disagreement. Since we only calculated percent agreement,
#the number that is selected does not influence the result as long as it differs from the score C1 gave.
funcs$singleC3 <- apply(funcs, 1, function(x){ifelse(x$singleC3==0, if(length(x$Therapy.C3)==1){x$Therapy.C3}else{sample(x$Therapy.C3,1)}, x$singleC3)})

#drop Therapy.C3 column as this is not needed anymore and rename the singleC3 column to Therapy.C3 
funcs$Therapy.C3 <- NULL
names(funcs)[names(funcs) == 'Therapy.C1'] <- 'C1'
names(funcs)[names(funcs) == 'singleC3'] <- 'C3'

#and write output to file again
write.table(funcs, "analysis/function_therapy_mapping_C1C3.txt",sep="\t", row.names = FALSE)