#set working directory to data folder
path <- "./preprocessing_and_raw_data/literature_screening/"

#########################################################################################################
# Preprocessing for analysis of reliability in filtering the literature to be included in the ehealth4mdd database. 
# 2018 TUDelft
# Author: Franziska Burger
# Output files: 
# - literature_screening_C1C2.txt.txt 
# - ii_papers_key_mapping.txt (for manual editing of papers included at title and abstract stage)
# Required files: 
# - title_exclude_dpl.txt
# - abstract_exclude_dpl.txt
# - abstract_include_dpl.txt 
# - all_refs_no_dpl.txt
# - papers_from_db.txt (exported from database, contains information on filtering of first author based on full title but papers have different keys than in endnote)
# - title_test_final_C2.txt
# - abstract_test_final_C2.txt
# - full_test_final_C2.txt
#########################################################################################################

################################# LIBRARIES
require(foreign)
require(car)
require(dplyr)
require(plyr)
require(tidyr)
require(reshape)
#################################

########################################################################### DATA ###################

########## title_exclude_dpl.txt, abstract_exclude_dpl.txt, abstract_include_dpl.txt
# title_exclude_dpl.txt: contains a list of all the records that were excluded by the first coder based on title, exported from endnote, contains some duplicate records
# abstract_exclude_dpl.txt: contains a list of all the records that were excluded by the first coder based on abstract, exported from endnote, contains some duplicate records
# abstract_include_dpl.txt: contains a list of all the records that were included by the first coder based on abstract, exported from endnote, contains some duplicate records
#
# data fields
#
# PaperKey: key of the document assigned by endnote upon import
# Filter: filtering information of first author (ee - exclude on title, ie - exclude on abstract, ii - include on abstract)
# Author: authors of record
# Year: year when record was published
# Title: title of record

########## all_refs_no_dpl.txt
# Contains all records without duplicates, exported from endnote 
#
# data fields
#
# PaperKey: key of the document assigned by endnote upon import
# Author: authors of record
# Year: year when record was published
# Title: title of record

########## papers_from_db.txt
# Contains all records that were included based on abstract by first coder as exported from the ehealth4mdd database
#
# data fields
#
# PaperKeyDB: key of the document assigned by database
# Year: year when record was published
# Title: title of record
# Database: origin of record (1- PubMed, 2-Scopus, 3-WebOfScience, 4-manually added), 4 has no corresponding key in endnote
# Exclude: 1 if excluded by first coder on the basis of the full paper, NA if included

########## title_test_final_C2.txt, abstract_test_final_C2.txt, full_test_final_C2.txt
# Contains codings of records that coder 2 (C2) coded on title, abstract, and full paper
#
# data fields
#
# Column1: record key
# Column2: coding of C2


########################################################################### FUNCTIONS ###################
#to match record numbers from the Endnote on computer of C1 with those from Endnote on computer of C2
overlap_synch <- function(c2df,c1df,stage){
  colnames(c1df)[2] <- "C1"
  c2df$V1 <- as.numeric(c2df$V1)
  c2df$V2 <- as.numeric(c2df$V2)
  c2df <- rename(c2df, c("V1"="PaperKey","V2"="C2"))
  overlap_df <- merge(c1df,c2df,by="PaperKey",all.y=TRUE)
  overlap_df$Stage <- stage
  return(overlap_df)
}

########################################################################### READING IN DATA ###################
########### Coding of Coder 1
#load all tables with duplicates but also with filtering info
titles_e <- read.table(file=paste0(path,"title_exclude_dpl.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)
abstract_e <- read.table(file=paste0(path,"abstract_exclude_dpl.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)
abstract_i <- read.table(file=paste0(path,"abstract_include_dpl.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)
#load table without duplicates and without filtering info
all_refs <- read.table(file=paste0(path,"all_refs_no_dpl.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)
#load records from database
papers <- read.table(file=paste0(path,"papers_from_db.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)
#manual editing of papers included at title and abstract stage
mapping_edited <- read.table(file=paste0(path,"ii_papers_key_mapping_edited.txt"), header=TRUE, sep="\t", quote="", fill=TRUE)

########### Coding of Coder 2
c2df.title <- read.table(file=paste0(path,"title_test_final_C2.txt"), header=FALSE, sep="\t",fill=TRUE)
c2df.abstract <- read.table(file=paste0(path,"abstract_test_final_C2.txt"), header=FALSE, sep="\t",fill=TRUE)
c2df.full <- read.table(file=paste0(path,"full_test_final_C2.txt"), header=FALSE, sep="\t",fill=TRUE)

########################################################################### PREPROCESSING ###################
########### Coding of Coder 1
#combine the three dataframes into one dataframe
all_refs_dpl <- rbind(rbind(titles_e, abstract_e), abstract_i)

#merge on the keys to remove duplicates but keep filtering info
all_refs_filter <- merge(all_refs_dpl[,c("PaperKey","Filter")], all_refs, by="PaperKey", all.y=TRUE)
all_refs_filter$Filter <- gsub(" ","",as.character(all_refs_filter$Filter)) #make Filter string format without spaces

#useful for analysis is also how the reference numbers from endnote are linked to records in DB
papers$Title <- as.character(papers$Title)
#replace 1 with iie (exclude based on full paper) and NA with iii (include based on full paper)
papers$Exclude <- ifelse(is.na(papers$Exclude),"iii","iie")

#now merge database papers and endnote papers based on title, so that we can obtain a mapping from database keys to endnote keys
all_refs$Title <- as.character(all_refs$Title)
mapping <- merge(papers[,c("PaperKeyDB","Title","Exclude")], all_refs[,c("PaperKey","Title")], by="Title", all.x=TRUE)

#generate second output file
write.table(mapping, paste0(path,"ii_papers_key_mapping.txt"), sep="\t")

#some records (approx. 5) could not be automatically matched on title, so this matching had to be done manually
#additionally, some duplicates are in the database. These were removed.
#now we make sure that our manual labor is correct by comparing titles
mapping_edited_check <- merge(mapping_edited, mapping[,c("PaperKeyDB","PaperKey")],by="PaperKeyDB")
mapping_edited_check <- merge(mapping_edited_check, all_refs[,c("PaperKey","Title")], by.x="PaperKey.x", by.y="PaperKey", all.x=TRUE)
#since all titles match, we can leave the edited file as is

#now we combine the two dataframes into one big dataframe with all filtering information of coder 1
filtering_c1 <- merge(all_refs_filter[,c("PaperKey","Filter")],mapping_edited[,c("PaperKey","Exclude")], by="PaperKey",all.x=TRUE)
filtering_c1$Full <- ifelse(filtering_c1$Exclude=="iii",1,0)
filtering_c1$Exclude <- ifelse(filtering_c1$Filter=="ii",1,ifelse(filtering_c1$Filter=="ee",NA,0))
filtering_c1$Filter <- ifelse(filtering_c1$Filter=="ee",0,1)
#finally we rename the columns and write the data to a file
filtering_c1 <- rename(filtering_c1, c("Full"="Full.C1", "Exclude"="Abstract.C1", "Filter"="Title.C1"))

########### Coding of Coder 2 merged with Codings of C1
#combine the ratings of C1 with C2 in one dataframe for title, abstract, and full-paper
overlap_tite.df <- overlap_synch(c2df.title,filtering_c1[,c("PaperKey","Title.C1")],"title")
overlap_abte.df <- overlap_synch(c2df.abstract,filtering_c1[,c("PaperKey","Abstract.C1")],"abstract")
overlap_fute.df <- overlap_synch(c2df.full,filtering_c1[,c("PaperKey","Full.C1")],"fullpaper")
litscreening.df <- rbind(rbind(overlap_tite.df,overlap_abte.df),overlap_fute.df)

write.table(litscreening.df, "analysis/literature_screening_C1C2.txt", sep="\t", row.names=FALSE)
